# This is the LDAP driver for squid-auth

# TODO: Implement some sort of LDAP caching to avoid
#       creating an LDAP object every time, and just
#       bind/unbind as necessary

# TODO: Uncomment the LDAPS 'use' statement when LDAPS or 
#       TLS can be used on Mandrake.  Other distros may be
#       fine, Mandrake 9.1 isn't.  Once that's done, the logic
#       should work fine.

use Net::LDAP;
#use Net::LDAPS;

sub __LDAP_connect
{
    my $host = $_[0];
    my $port = $_[1];
    my $dn = $_[2];
    my $passwd = $_[3];
    my $config = $_[4];

    my $protocol = $config->{'protocol'};
    $protocol = "ldap" unless defined( $protocol );

    my $is_tls = 0;
    my $is_encrypted = 0;

    my $client_cert = $config->{'cert'};
    my $client_key = $config->{'key'};
    my $ca_cert = $config->{'cacert'};
    my $ca_path = $config->{'capath'};
    my $verify = $config->{'verify'};
    $client_key = $client_cert unless defined( $client_key );

    debug( 1, "Binding to the LDAP server" );

    if ( $protocol eq "ldap" )
    {
        $port = 389 unless defined( $port );
        if ( defined( $client_cert ) or defined( $ca_cert ) or defined( $ca_path ) or defined( $verify ) )
        {
            $is_tls = 1;
            $is_encrypted = 1;
        }
    }
    elsif ( $protocol eq "ldaps" )
    {
        $port = 636 unless defined( $port );
        $is_tls = 0;
        $is_encrypted = 1;
    }
    else
    {
        return error( "Unsupported LDAP Protocol variant '$protocol'" );
    }

    my %conn_param = ();
    my %sec_param = ();

    $conn_param{version} = 3;

    if ( $is_encrypted )
    {
        if ( defined( $client_cert ) )
        {
            ( -f $client_cert ) or return error( "Client certificate '$client_cert' not found" );
            $sec_param{clientcert} = $client_cert;
            $sec_param{clientkey} = $client_key;
        }

        if ( defined( $verify ) )
        {
            $sec_param{verify} = $verify;
        }
        $verify = "none" unless defined( $verify );

        if ( defined( $ca_cert ) or defined( $ca_path ) )
        {
            if ( defined( $ca_path ) )
            {
                ( -d $ca_path ) or return error( "CA path '$ca_path' not found" );
                $sec_param{capath} = $ca_path;
            }
            else
            {
                ( -f $ca_cert ) or return error( "CA certificate '$ca_cert' not found" );
                $sec_param{cafile} = $ca_cert;
            }
            $verify = "optional" unless ( $verify eq "required" );
        }

        $sec_param{verify} = $verify;
        unless ( $is_tls )
        {
            $sec_param{port} = $port;
        }
        else
        {
            debug( 3, "TLS - requiring LDAPv3" );
            $conn_param{port} = $port;
        }
    }
    else
    {
        $conn_param{port} = $port;
    }

    my $ldap;

    if ( $is_tls or ! $is_encrypted )
    {
        debug( 3, "LDAP: ( $host, $port )" );
        $ldap = new Net::LDAP( $host, %conn_param );
    }
    else
    {
        debug( 3, "LDAPS: ( $host, $port )" );
        $ldap = new Net::LDAPS( $host, %sec_param );
    }
    return ( "Unable to create new LDAP object" ) unless defined( $ldap );

    if ( $is_tls )
    {
        debug( 3, "LDAP_TLS: Starting" );
        my $tls = $ldap->start_tls( %sec_param );
        if ( $tls->code() )
        {
            my $err = $tls->error();
            $ldap->unbind();
            return error( "$err" );
        }
    }

    debug( 3, "BIND: ( $dn, <passwd> )" );
    my $bind = $ldap->bind( $dn, password => $passwd );
    if ( $bind->code() )
    {
        my $err = $bind->error;
        $ldap->unbind();
        return error( "$err" );
    }

    $ldap->unbind();
    return 0;
}

sub authenticate_user
{
    my $config = $_[0];
    my $mod_home = $_[1];
    my $uid = $_[2];
    my $realm = $_[3];
    my $passwd = $_[4];

    # First, make sure we have the config parameters we need
    my $host = $config->{'host'};
    defined( $host ) or return error( "No host specified in $realm\'s configuration" );

    my $dn = $config->{'dnstring'};
    defined( $dn ) or return error( "No 'dnstring' parameter in $realm\'s configuration" );

    # Now, perform the substitution on the DN string
    while ( $dn =~ /%{.*}/ )
    {
        $dn =~ s/%{uid}/${uid}/g;
        $dn =~ s/%{realm}/${realm}/g;
    }

    my $port = $config->{'port'};
    return __LDAP_connect( $host, $port, $dn, $passwd, $config );
}

1;
